<?php

/*
 * Defines some token module enabled actions & conditions
 */


/*
 * Implementation of hook_condition_info()
 */
function workflow_ng_ui_condition_info() {
  return array(
    'workflow_ng_condition_user_comparison' => array(
      '#label' => t('Compare two users'),
      '#arguments' => array(
        'user1' => array('#entity' => 'user', '#label' => t('First user to compare')),
        'user2' => array('#entity' => 'user', '#label' => t('Second user to compare'))
      ),
      '#description' => t('Evaluates to TRUE, if both compared users are the same user account.'),
      '#module' => t('User'),
    ),
    'workflow_ng_condition_token_compare' => array(
      '#label' => t('Textual comparison'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#description' => t('This condition just compares two texts. It returns TRUE, if both texts are equal.').
        ' '. t('You may use token replacements in both texts, e.g. the texts %text1 and %text2 are only equal, if the argument called "node" is of the type page.', array('%text1' => "[node:type]", '%text2' => 'page')),
      '#module' => t('Token'),
    ),
  );
}

/*
 * Implementation of hook_action_info()
 */
function workflow_ng_ui_action_info() {
  return array(
    'workflow_ng_action_mail_to_user' => array(
      '#label' => t('Send a mail to a user'),
      '#arguments' => array(
        'user' => array('#entity' => 'user', '#label' => t('Recipient')),
      ),
      '#module' => t('Token'),
      '#description' => t('You may use the token replacements for the mail\'s message body as well as the mail\'s subject.'),
    ),
    'workflow_ng_action_node_set_author' => array(
      '#label' => t('Set the content author'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('Content')),
        'author' => array('#entity' => 'user', '#label' => t('User, which is set as author')),
      ),
      '#module' => t('Node'),
    ),
    'workflow_ng_action_drupal_message' => array(
      '#label' => t('Show a configurable message on the site'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('Token'),
    ),
    'workflow_ng_action_drupal_goto' => array(
      '#label' => t('Page redirect'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('Token'),
    ),
  );
}


/*
 * Action Implementation: Show a configureable message 
 */
function workflow_ng_action_drupal_message($settings, &$arguments, &$log) {
  $message = workflow_ng_token_replace($settings['message'], $settings['used arguments'], $arguments, $log);
  drupal_set_message($message);
}

/*
 * Action "Show a configureable message" configuration form
 */
function workflow_ng_action_drupal_message_form($settings = array(), $argument_info) {

  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The message that should be displayed.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_drupal_message_submit($form_id, $form_values) {
  //returns the needed settings
  $used_args = workflow_ng_token_get_used_arguments($form_values['message'], $form_values['arguments']);
  return array('message' => $form_values['message'], 'used arguments' => $used_args);
}

/*
 * Action Implementation: Page redirect
 */
function workflow_ng_action_drupal_goto($settings, &$arguments, &$log) {
  $path = workflow_ng_token_replace($settings['path'], $settings['path args'], $arguments, $log);
  $destination = '';
  if ($settings['force']) {
    $destination = 'destination='. urlencode($_REQUEST['destination']);
    unset($_REQUEST['destination']);
  }
  drupal_goto($path, $destination);
}

/*
 * Action "Page redirect" configuration form
 */
function workflow_ng_action_drupal_goto_form($settings = array(), $argument_info) {
  $form['path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path'),
    '#default_value' => $settings['path'],
    '#description' => t('The path, to which the user is to be redirected.'),
  );
  $form['force'] = array(
    '#type' => 'checkbox',
    '#title' => t('Force redirecting to the given path, even if a destination paramter is given'),
    '#description' => t('Per default drupal doesn\'t redirect to the given path, if a destination parameter is set.'.
      'Instead it redirects to the given destination paramter. Most times, the destination parameter is set by appending it to the URL, e.g. !example_url', array('!example_url' => 'http://example.com/user/login?destination=node/2')),
    '#default_value' => $settings['force'],
    '#weight' => 7,
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_drupal_goto_submit($form_id, $form_values) {
  //returns the needed settings
  $used_args = workflow_ng_token_get_used_arguments($form_values['path'], $form_values['arguments']);
  return array('path' => $form_values['path'], 'force' => $form_values['force'], 'path args' => $used_args);
}


/*
 * Action Implementation: Send a mail to a user
 */
function workflow_ng_action_mail_to_user($settings, $user, &$arguments, &$log) {
  $subject = workflow_ng_token_replace($settings['subject'], $settings['subject_args'], $arguments, $log);
  $subject = str_replace(array("\r", "\n"), '', $subject);
  $message = workflow_ng_token_replace($settings['message'], $settings['msg_args'], $arguments, $log);
  if (drupal_mail('workflow_ng_action_mail_to_user', $user->mail, $subject, $message)) {
    watchdog('action', t('Sent email to %recipient', array('%recipient' => $user->name)), WATCHDOG_NOTICE, theme('username', $user));
  }
  else {
    watchdog('error', t('Unable to send email to %recipient', array('%recipient' => $user->name)), WATCHDOG_NOTICE, theme('username', $user));
  }
}

/*
 * Action "Send a mail to a user" configuration form
 */
function workflow_ng_action_mail_to_user_form($settings = array(), $argument_info) {
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#description' => t('The mail\'s subject.'),
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The mail\'s message body.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_mail_to_user_submit($form_id, $form_values) {
  return array(
    'message' => $form_values['message'],
    'subject' => $form_values['subject'],
    'msg_args' => workflow_ng_token_get_used_arguments($form_values['message'], $form_values['arguments']),
    'subject_args' => workflow_ng_token_get_used_arguments($form_values['subject'], $form_values['arguments']),
  );
}


/*
 * Condition Implementation: Token enabled comparison 
 */
function workflow_ng_condition_token_compare($settings, &$arguments, &$log) {
  $text1 = workflow_ng_token_replace($settings['text1'], $settings['used arguments1'], $arguments, $log);
  $text2 = workflow_ng_token_replace($settings['text2'], $settings['used arguments2'], $arguments, $log);
  return $text1 == $text2;
}

/*
 * Condition: Token enabled comparison configuration form
 */
function workflow_ng_condition_token_compare_form($settings = array(), $argument_info) {
  $form = array();
  $form['text1'] = array(
    '#type' => 'textfield',
    '#title' => 'Two texts to compare',
    '#default_value' => isset($settings['text1']) ? $settings['text1'] : '',
  );
  $form['text2'] = array(
    '#type' => 'textfield',
    '#title' => '',
    '#default_value' => isset($settings['text2']) ? $settings['text2'] : '',
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_condition_token_compare_submit($form_id, $form_values) {
  //returns the needed settings
  $settings = array();
  foreach (array(1,2) as $i) {
    $settings['used arguments'. $i] = workflow_ng_token_get_used_arguments($form_values['text'. $i], $form_values['arguments']);
    $settings['text'. $i] = $form_values['text'. $i];
  }
  return $settings;
}


/*
 * Returns an array of all used arguments in $text
 * 
 * @param $text The $text, which will be processed
 * @param $argument_info The argument definitions we work with
 * @return The array of names of the used arguments
 */
function workflow_ng_token_get_used_arguments($text, $argument_info) {
  $used_args = array();
  foreach ($argument_info as $name => $argument) {
    if (strpos($text, '['. $name . ':') !== FALSE) {
      $used_args[] = $name;
    }
  }
  return $used_args;
}

/*
 * Performs the replacement of the replacement patterns.
 * If a used argument isn't loaded yet, it will be automatically loaded by workflow_ng_element_get_argument()
 * 
 * @param $text The $text, which will be processed
 * @param $used_arguments The array of names of used arguments
 * @param $arguments The workflow-ng arguments data structure
 * @param $log An array of log entries. Set it to FALSE to disable logging
 * @return The text with replaced patterns.
 */
function workflow_ng_token_replace($text, $used_arguments, &$arguments, &$log) {
  static $token_cache = array();

  foreach($used_arguments as $name) {
    $data = workflow_ng_element_get_argument($arguments, $name, $log);
    $entity = $arguments['info'][$name]['#entity'];

    $token_id = _token_get_id($entity, $data);
    if (isset($token_cache[$token_id]) && $token_cache[$token_id] != $name) {
      //this is another argument although it has the same id
      //so we need to flush the token cache manually 
      token_get_values('global', NULL, TRUE);
    }

    $text = token_replace($text, $entity, $data, '['. $name . ':', ']');

    //remember that this argument has been used and token has cached it...
    $token_cache[$token_id] = $name;
  }
  return $text;
}

/*
 * Adds the token replacement help to $form
 */
function workflow_ng_token_replacement_help(&$form, $argument_info) {
  if (isset($argument_info) && count($argument_info)) {
    $form['replace'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#description' => t('You can make use of the replacement patterns of all available arguments.'),
    );
    foreach ($argument_info as $name => $argument) {
      $form['replace'][$name] = array(
        '#type' => 'fieldset',
        '#title' => t('Replacement patterns for @name', array('@name' => t($argument['#label']))),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['replace'][$name]['content'] = array(
        '#value' => theme('token_help', $argument['#entity'], '['. $name . ':', ']'),
      );
    }
  }
  $form['arguments'] = array('#type' => 'value', '#value' => $argument_info);
}


/*
 * A simple user comparison
 */
function workflow_ng_condition_user_comparison($user1, $user2) {
  return $user1->uid == $user2->uid;
}

/*
 * Modifies a node as configured
 */
function workflow_ng_action_node_set_author($node, $author) {
  $node->uid = $author->uid;
  $node->name = $author->name;
  return array('node' => $node);
}


/*
 * Dummy action: mail a node to the user
 */
function workflow_ng_action_mail_node($node, $user) {
  //actually this action doesn't send any mail.., that's why it is called dummy.. ;)
  drupal_set_message(t("Informed %name about your changes.", array('%name' => $user->name)));
}
